import 'dart:convert';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:contacts_service/contacts_service.dart';
import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:http/http.dart' as http;
import 'package:flutter/material.dart';
import 'package:mini_app_likewallet/config/config.dart';
import 'package:mini_app_likewallet/library/auth.dart';
import 'package:mini_app_likewallet/library/configuration_service.dart';
import 'package:mini_app_likewallet/library/ethcontract.dart';
import 'package:mini_app_likewallet/middleware/check_auth.dart';
import 'package:mini_app_likewallet/middleware/get_mnemonic.dart';
import 'package:mini_app_likewallet/model/contact.dart';
import 'package:permission_handler/permission_handler.dart';

class SendProvider with ChangeNotifier {
  List<String> sendAmount = [];
  TextEditingController noteController = TextEditingController();
  String nameTo = '';
  String phone = '';
  String addressTo = '';
  String nameFOMR = '';
  bool loading = false;
  bool loadingContact = false;
  bool tapAddNote = false;
  bool confirm = false;
  bool contract = false;
  double rateCurrency = 0.0;
  List<Contacts> listContact = [];
  Iterable<Contact> contacts;
  SendProvider();

  List<String> get getAmount => sendAmount;
  bool get getTapAddNote => tapAddNote;
  bool get getConfirm => confirm;
  bool get getContract => contract;
  String get getNameTo => nameTo;
  String get getNameFOMR => nameFOMR;
  String get getPhone => phone;
  bool get getLoading => loading;
  bool get getLoadingContact => loadingContact;
  String get getAddressTo => addressTo;
  double get getRateCurrency => rateCurrency;
  List<Contacts> get getListContact => listContact;

  BaseAuth auth;
  Dio dio = new Dio();

  setLoading({bool status}) {
    loading = status;
    notifyListeners();
  }

  setLoadingContact({bool status}) {
    loadingContact = status;
    notifyListeners();
  }

  changeAmount({var value}) {
    sendAmount += value;
    notifyListeners();
  }

  deleteAmount() {
    if (sendAmount.length >= 1) {
      sendAmount.removeLast();
    }
    notifyListeners();
  }

  setTextAmount({var number}) {
    sendAmount.add(number.toString());
    notifyListeners();
  }

  changeAddNote({bool status}) {
    tapAddNote = status;
    notifyListeners();
  }

  changeConfirm({bool status}) {
    confirm = status;
    notifyListeners();
  }

  changeContact({bool status}) {
    contract = status;
    notifyListeners();
  }

  setListContact({List<Contacts> data}) {
    listContact = data;
    notifyListeners();
  }

  void getNameTO({String address}) async {
    auth = Auth();
    addressTo = address;
    loading = true;
    final _token = await auth.getTokenFirebase();
    try {
      Response response = await dio.post(Env.apiUrl + "/searchNameNew",
          data: {"term": address, "_token": _token});
      if (response.data["statusCode"] == 200) {
        print(response.data["result"]['name']);
        print(response.data["result"]);
        nameTo = response.data["result"]['name'];
        phone = response.data["result"]['phoneNumber'];
        loading = false;
        notifyListeners();
      }
    } catch (e) {
      print(e);
    }
  }

  void getNameFROM({String address}) async {
    auth = Auth();
    final _token = await auth.getTokenFirebase();
    try {
      Response response = await dio.post(Env.apiUrl + "/searchNameNew",
          data: {"term": address, "_token": _token});
      if (response.data["statusCode"] == 200) {
        print(response.data["result"]['name']);
        print(response.data["result"]);
        nameFOMR = response.data["result"]['name'];
        notifyListeners();
      }
    } catch (e) {
      print(e);
    }
  }

  Future<bool> statusPermission() async {
    var status = await Permission.photos.status;
    final permissionStatus = await Permission.photos.request();
    if (permissionStatus == PermissionStatus.granted) {
      return true;
    } else if (status.isDenied ||
        status.isUndetermined ||
        status.isPermanentlyDenied) {
      return false;
    }
  }

  getRateCurrencySymbol({String symbol}) {
    FirebaseFirestore.instance
        .collection('exchangeFiat')
        .doc(symbol == 'none' ? "THB-LIKE" : "THB-" + symbol)
        .get()
        .then((DocumentSnapshot ds) {
      print(ds.data()["rate"]);
      if (ds.data()["to"] == 'LIKE') {
        rateCurrency = ds.data()["rate"];
      } else {
        rateCurrency = 1 / ds.data()["rate"];
        print(rateCurrency);
      }
    });
    notifyListeners();
  }

  Future<String> signTransaction({
    IConfigurationService configETH,
    EthContract eth,
    CurCheckAuth login,
    GetMnemonic seed,
    String addressTo,
    String amount,
    String contract,
    String callFunction,
    String abi,
  }) async {
    double rate = 100;
    String tx = '';
    num avaiBalance = await eth.getBalance(address: configETH.getAddress());
    if ((avaiBalance >= double.parse(amount) * rateCurrency * rate)) {
      //เช็ต login
      bool loginStatus = await login.checkLogin();
      if (loginStatus) {
        //เช็ต Seed กระเป๋า
        String value = await seed.getMnemonic();
        if (value.isNotEmpty) {
          // final mnemonic = value.split(":")[0];
          final pkEth = value.split(":")[1];

//        eth.sendTransaction(pk: pketh, to: nameTO.trim(), value: (double.parse(amount)*rateCurrency*rate).toString()).then((tx) {
          if (contract == 'no') {
            final resultTX = await eth.transferMessage(
                pk: pkEth,
                to: addressTo.trim(),
                value: (double.parse(amount) * rateCurrency * rate).toString(),
                message: noteController.text.toString());

            tx = resultTX;
            print('tx : $tx');
          } else if (contract.length == 42 &&
              callFunction != 'no' &&
              abi != 'no') {
            final resultTX = await eth.dynamicSendContract(
                pk: pkEth,
                to: addressTo.trim(),
                value: (double.parse(amount) * rateCurrency * rate).toString(),
                message: noteController.text.toString(),
                callFunction: callFunction,
                abi: abi,
                contractAddress: contract);
            tx = resultTX;
            print('tx : $tx');
          }
        }
      }
    } else {
      print('คุณมี LIKE ไม่เพียงพอ');
      // showShortToast(AppLocalizations.of(context).translate('not_enough') +
      //     AppLocalizations.of(context).translate('not_enough_tail') +
      //     " " +
      //     (avaiBalance).toString() +
      //     "LIKE");
    }
    return tx;
  }

  Future<PermissionStatus> permissionContact() async {
    final PermissionStatus permission = await Permission.contacts.status;
    if (permission != PermissionStatus.granted &&
        permission != PermissionStatus.denied) {
      final Map<Permission, PermissionStatus> permissionStatus =
          await [Permission.contacts].request();
      return permissionStatus[Permission.contacts] ??
          PermissionStatus.undetermined;
    } else {
      return permission;
    }
  }

  Future<List<Contacts>> getContacts({BuildContext context}) async {
    setLoadingContact(status: true);
    List phoneContactTH = [];
    List phoneContactLAO = [];
    List phoneContactKM = [];
    List<Contacts> getContactMobile = [];
    final PermissionStatus permissionStatus = await permissionContact();
    if (permissionStatus == PermissionStatus.granted) {
      contacts = await ContactsService.getContacts();

      contacts.forEach((element) async {
        element.phones.forEach((phone) async {
          if (phone.value.toString().substring(0, 1) == '0') {
            final p = phone.value.toString();
            phoneContactTH.add(p.replaceFirst('0', '+66').replaceAll(' ', ''));
            phoneContactLAO
                .add(p.replaceFirst('0', '+885').replaceAll(' ', ''));
            phoneContactKM.add(p.replaceFirst('0', '+888').replaceAll(' ', ''));
          } else {}
        });
      });

      print(phoneContactTH);
      for (int i = 0; i < phoneContactTH.length; i++) {
        QuerySnapshot th = await FirebaseFirestore.instance
            .collection('addressDNS')
            .where('phoneNumber', isEqualTo: phoneContactTH[i])
            .get();
        if (th.docs.isNotEmpty) {
          print(th);
          listContact.add(Contacts.fromJson(th.docs[0].data()));
        }
      }
      notifyListeners();
      for (int i = 0; i < phoneContactLAO.length; i++) {
        QuerySnapshot loa = await FirebaseFirestore.instance
            .collection('addressDNS')
            .where('phoneNumber', isEqualTo: phoneContactLAO[i])
            .get();
        if (loa.docs.isNotEmpty) {
          print(loa);
          listContact.add(Contacts.fromJson(loa.docs[0].data()));
        }
      }
      notifyListeners();
      for (int i = 0; i < phoneContactKM.length; i++) {
        QuerySnapshot km = await FirebaseFirestore.instance
            .collection('addressDNS')
            .where('phoneNumber', isEqualTo: phoneContactKM[i])
            .get();
        if (km.docs.isNotEmpty) {
          print(km);
          listContact.add(Contacts.fromJson(km.docs[0].data()));
        }
      }
      loadingContact = false;
      notifyListeners();
      if (listContact.length == 0) {
        listContact = null;
      }
    } else {
      //If permissions have been denied show standard cupertino alert dialog
      showDialog(
          context: context,
          builder: (BuildContext context) => CupertinoAlertDialog(
                title: Text('Permissions error'),
                content: Text('Please enable contacts access '
                    'permission in system settings'),
                actions: <Widget>[
                  CupertinoDialogAction(
                    child: Text('OK'),
                    onPressed: () => Navigator.of(context).pop(),
                  )
                ],
              ));
      // return '';
    }
    return getContactMobile;
  }
}
