import 'dart:io';
import 'dart:math';
import 'dart:ui';

import 'package:device_apps/device_apps.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_appavailability/flutter_appavailability.dart';

import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:launch_review/launch_review.dart';
import 'package:mini_app_likewallet/device_utils.dart';
import 'package:mini_app_likewallet/home/home_component/home_component.dart';
import 'package:mini_app_likewallet/library/address_service.dart';
import 'package:mini_app_likewallet/library/configuration_service.dart';
import 'package:mini_app_likewallet/library/ethcontract.dart';
import 'package:mini_app_likewallet/middleware/check_auth.dart';
import 'package:mini_app_likewallet/middleware/get_mnemonic.dart';
import 'package:mini_app_likewallet/provider/home_provider/check_tx_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/home_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/send_provider/send_provider.dart';
import 'package:mini_app_likewallet/send/component/send_confirm_component.dart';
import 'package:mini_app_likewallet/theme/color.dart';
import 'package:mini_app_likewallet/theme/custom_loading.dart';
import 'package:mini_app_likewallet/theme/image.dart';
import 'package:mini_app_likewallet/theme/loading.dart';
import 'package:mini_app_likewallet/theme/widget.dart';
import 'package:modal_progress_hud/modal_progress_hud.dart';
import 'package:open_appstore/open_appstore.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:mini_app_likewallet/model/contact.dart';

class SendConfirmScreen extends StatefulWidget {
  @override
  _SendConfirmScreen createState() => _SendConfirmScreen();
}

class _SendConfirmScreen extends State<SendConfirmScreen> {
  SendProvider _sendProvider;
  CheckTXProvider _checkTXProvider;
  HomeProvider _getLikeWallet;
  IAddressService addressService;
  IConfigurationService configETH;
  CurCheckAuth login;
  BaseETH eth;
  GetMnemonic seed;
  String mnemonic;
  double balanceLIKE = 0.00;
  List<Contacts> _contacts;

  Random random = new Random();
  List colors = [
    Color(0xff8694FF),
    Color(0xff2AE5D6),
    Color(0xffADE000),
    Color(0xffBB89FF)
  ];
  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();
    _getLikeWallet = Provider.of<HomeProvider>(context);
    _sendProvider = Provider.of<SendProvider>(context, listen: true);
    _checkTXProvider = Provider.of<CheckTXProvider>(context, listen: true);
    if (_sendProvider.getNameTo != '' && _sendProvider.getAmount != null) {
      _sendProvider.confirm = true;
      _sendProvider.contract = false;
    }
    if (_sendProvider.getAmount != null && _sendProvider.getNameTo == '') {
      _sendProvider.confirm = false;
      _sendProvider.contract = true;
    }
  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    login = new CurCheckAuth();
    eth = new EthContract();
    seed = new MnemonicRetrieve();
    setInitFirst();
  }

  setInitFirst() async {
    SharedPreferences pref = await SharedPreferences.getInstance();
    _sendProvider.getContacts(context: context);
    configETH = new ConfigurationService(pref);
    addressService = new AddressService(configETH);
    mnemonic = await configETH.getMnemonic();
    if (mnemonic == 'no') {
      print('ดีดไป Login');
    }
    // fiat = pref.getString('currency') ?? 'none';
    // print('fiat : ' + fiat);
    String getPK = addressService.getPrivateKey(mnemonic);
    configETH.setPrivateKey(getPK);
    addressService.getPublicAddress(getPK).then((address) {
      print(address.toString());
      configETH.setAddress(address.toString());
      eth.getBalance(address: address.hex).then((balance) {
        balanceLIKE = balance;
      });
    });
    // final contacts = await contact.getContacts(context: context);
  }

  @override
  Widget build(BuildContext context) {
    ScreenUtil.init(context,
        designSize: Size(1080, 2340), allowFontScaling: false);
    return ModalProgressHUD(
      opacity: 0.1,
      inAsyncCall: _sendProvider.getLoading,
      progressIndicator: CustomLoading(),
      child: Scaffold(
//      backgroundColor: Colors.black.withOpacity(0.1),
        body: GestureDetector(
          onTap: () {
            DeviceUtils.hideKeyboard(context);
          },
          child: Container(
            color: Colors.transparent,
            height: 1491.h,
            child: Stack(
              children: [
                AnimatedPositioned(
                    duration: Duration(milliseconds: 150),
                    top: _sendProvider.getContract ? 729.h : 0.h,
                    child: BackdropFilter(
                        filter: ImageFilter.blur(sigmaX: 0, sigmaY: 0),
                        child: sendContactBackground())),
                AnimatedPositioned(
                    duration: Duration(milliseconds: 150),
                    top: _sendProvider.getConfirm ? 729.h : 0.h,
                    child: BackdropFilter(
                        filter: ImageFilter.blur(sigmaX: 0, sigmaY: 0),
                        child: sendConfirmBackground())),
                Container(
                  color: Colors.transparent,
                  height: 1064.1.h,
                  child: Stack(children: [
                    Positioned(
                        top: -0.h,
                        child: SendConfirmComponent.sendBackground()),
                    Column(children: [
                      SizedBox(height: 140.h),
                      Container(
                          height: 101.56.h,
                          margin: EdgeInsets.only(left: 75.w),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              HomeComponent.nameWallet(
                                  name: _sendProvider.nameFOMR),
                              InkWell(
                                  onTap: () => _launchURL(),
                                  child: HomeComponent.openLikeWallet())
                            ],
                          )),
                      SizedBox(height: 139.9.h),
                      SendConfirmComponent.sendTotal(
                          amount: _sendProvider.getAmount.join()),
                      SizedBox(height: 89.h),
                      InkWell(
                          onTap: () {},
                          child: SendConfirmComponent.sendTo(context: context)),
                      SizedBox(height: 95.h),
                      InkWell(
                          onTap: () =>
                              _sendProvider.changeAddNote(status: true),
                          child: _sendProvider.getTapAddNote
                              ? SendConfirmComponent.note(
                                  noteController: _sendProvider.noteController)
                              : SendConfirmComponent.addNote()),
                      Expanded(child: Container()),
                      InkWell(
                          onTap: () {
                            _getLikeWallet.setSendConfirm(status: false);
                            _getLikeWallet.setSend(status: true);
                            _sendProvider.sendAmount = [];
                            _sendProvider.listContact = [];
                            _sendProvider.nameTo = '';
                            _sendProvider.phone = '';
                            _sendProvider.changeConfirm(status: false);
                            _sendProvider.changeContact(status: false);
                          },
                          child: SendConfirmComponent.backButton()),
                      SizedBox(height: 90.1.h),
                    ]),
                  ]),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget sendContactBackground() => Container(
      decoration: new BoxDecoration(
          color: Colors.white.withOpacity(0.8),
          borderRadius: new BorderRadius.all(Radius.circular(40.0))),
      height: 762.h,
      width: 1080.w,
      child: Column(
        children: [
          SizedBox(height: 402.h),
          Container(
              margin: EdgeInsets.only(left: 75.h - 24.h),
              height: 300.h,
              // width: MediaQuery.of(context).size.width - 254.w,
              child: _sendProvider.loadingContact
                  //Build a list view of all contacts, displaying their avatar and
                  // display name
                  ? Center(child: Loading.showLoading(context: context))
                  : _sendProvider.listContact == null
                      ? Text('ไม่พบรายชื่อ')
                      : ListView.builder(
                          // physics: NeverScrollableScrollPhysics(),
                          itemCount: _sendProvider.listContact?.length ?? 0,
                          scrollDirection: Axis.horizontal,
                          itemBuilder: (BuildContext context, int index) {
                            // Contact contact = _contacts?.elementAt(index);
                            return InkWell(
                              onTap: () {
                                if (_sendProvider.listContact[index].address !=
                                    '') {
                                  _sendProvider.setLoading(status: true);
                                  _sendProvider.getNameTO(
                                      address: _sendProvider
                                          .listContact[index].address);
                                } else {
                                  print('ไม่พบ Address');
                                  // showShortToast('ไม่พบ Address', Colors.red);
                                }
                              },
                              child: Container(
                                margin: EdgeInsets.only(
                                    left: 24.w, right: 24.w, top: 24.h),
                                // width: 155.w,
                                height: MediaQuery.of(context).size.height,
                                child: Column(
                                  children: [
                                    Container(
                                      height: 144.h,
                                      width: 155.w,
                                      child: Stack(
                                        children: [
                                          Container(
                                            height: 132.h,
                                            width: 155.w,
                                            alignment: Alignment.center,
                                            decoration: BoxDecoration(
                                              color: colors[random.nextInt(3)],
                                              borderRadius:
                                                  BorderRadius.circular(
                                                      100.0.h),
                                            ),
                                            child: Text(_sendProvider
                                                .listContact[index].name
                                                .toString()
                                                .substring(0, 1)),
                                          ),
                                          Positioned(
                                            bottom: 0,
                                            right: 12.w,
                                            child: Image.asset(
                                              ImageTheme.iconLikeWallet,
                                              package: 'mini_app_likewallet',
                                              height: 60.h,
                                              width: 60.h,
                                            ),
                                          )
                                        ],
                                      ),
                                    ),
                                    SizedBox(height: 21.h),
                                    Row(
                                      children: [
                                        Container(
                                          width: 155.h,
                                          child: Text(
                                            _sendProvider
                                                .listContact[index].name,
                                            style: TextStyle(
                                              fontFamily: '',
                                              fontSize: 27.h,
                                              color: const Color(0xcc3c3c43),
                                              letterSpacing:
                                                  0.8099999999999999.w,
                                              height: 1.1111111111111112,
                                            ),
                                            textHeightBehavior:
                                                TextHeightBehavior(
                                                    applyHeightToFirstAscent:
                                                        false),
                                            textAlign: TextAlign.center,
                                          ),
                                        ),
                                        // Text(
                                        //   _contacts[index].phoneNumber,
                                        //   style: TextStyle(
                                        //     fontFamily: AppLocalizations.of(context)
                                        //         .translate('font1'),
                                        //     fontSize: 27.h,
                                        //     color: const Color(0xcc3c3c43),
                                        //     letterSpacing: 0.8099999999999999.w,
                                        //     height: 1.1111111111111112,
                                        //   ),
                                        //   textHeightBehavior: TextHeightBehavior(
                                        //       applyHeightToFirstAscent: false),
                                        //   textAlign: TextAlign.center,
                                        // ),
                                      ],
                                    )
                                  ],
                                ),
                              ),
                            );
                          },
                        ))
        ],
      ));

  Widget sendConfirmBackground() => Container(
        decoration: new BoxDecoration(
            color: Colors.white.withOpacity(0.8),
            borderRadius: new BorderRadius.all(Radius.circular(40.0))),
        height: 762.h,
        width: 1080.w,
        child: Column(
          children: [
            SizedBox(height: 381.h),
            InkWell(
                onTap: () async {
                  _sendProvider.setLoading(status: true);
                  SharedPreferences pref =
                      await SharedPreferences.getInstance();
                  _sendProvider.getNameFROM(
                      address: pref.getString('addressETH'));
                  // print(double.parse(
                  //     _sendProvider.sendAmount.join().replaceAll(',', '')));
                  // print(balanceLIKE);

                  if (double.parse(_sendProvider.sendAmount
                          .join()
                          .replaceAll(',', '')) <=
                      balanceLIKE) {
                    await _sendProvider.getRateCurrencySymbol(symbol: 'none');
                    final tx = await _sendProvider.signTransaction(
                      configETH: configETH,
                      amount:
                          _sendProvider.sendAmount.join().replaceAll(',', ''),
                      eth: eth,
                      login: login,
                      seed: seed,
                      abi: 'no',
                      callFunction: 'no',
                      contract: 'no',
                      addressTo: _sendProvider.getAddressTo,
                    );
                    if (tx.isNotEmpty) {
                      final status =
                          await _checkTXProvider.checkStatusTX(tx: tx);
                      if (status) {
                        _sendProvider.setLoading(status: false);
                        _getLikeWallet.setSendReceipt(status: true);
                      }
                    } else {
                      print('tx ไม่มา');
                    }
                  } else {
                    print('ยอดเงินไม่พอ');
                  }
                  // _getLikeWallet.setSendReceipt(status: true);
                  // print(_getLikeWallet.getSendReceipt);
                },
                child: SendConfirmComponent.buttonConfirm()),
            SizedBox(height: 28.h),
            InkWell(
                onTap: () {
                  _sendProvider.nameTo = '';
                  _sendProvider.phone = '';
                  _sendProvider.changeContact(status: true);
                  _sendProvider.changeConfirm(status: false);
                },
                child: SendConfirmComponent.buttonCancel()),
          ],
        ),
      );
  Widget iconLike() => Image.asset(
        ImageTheme.iconLike,
        height: 84.37.h,
        width: 84.37.w,
        package: 'mini_app_likewallet',
      );
  _openURLAPP(url) async {
    print(url);
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      LaunchReview.launch(androidAppId: url, iOSAppId: url);
      throw 'Could not launch $url';
    }
  }

  _launchURL() async {
    String url;
    //ตรวจสอบ platform
    if (Platform.isAndroid) {
      url = 'com.prachakij.ldx';
      //ตรวจสอบว่ามีการติดตั้งไว้ในเครื่องหรือยัง
      bool isInstalled = await DeviceApps.isAppInstalled(url);
      //ถ้าติดตั้งแอพแล้วให้เปิดเลย
      if (isInstalled) {
        print('open app');
        await DeviceApps.openApp(url);
      } else {
        //ถ้ายังไม่มีให้ไปเปิดใน play store เพื่อติดตั้งแทน
        print('ถ้ายังไม่มีให้ไปเปิดใน');
        _openURLAPP(url);
      }
      //ถ้าเป็น iOS
    } else if (Platform.isIOS) {
      String os = Platform.operatingSystem;

      if (os == "ios") {
        print("Container clicked");
        try {
          await launch("ldx://");
        } on PlatformException catch (e) {
          OpenAppstore.launch(
              androidAppId: "com.prachakij.ldx", iOSAppId: "1504852835");
        } finally {
          await launch("ldx://");
          OpenAppstore.launch(
              androidAppId: "com.prachakij.ldx", iOSAppId: "1504852835");
//        launch("https://apps.apple.com/us/app/id1504852835");
        }
      } else if (os == "android") {
        try {
          await AppAvailability.launchApp("com.prachakij.ldx");
        } catch (e) {
          const url =
              'https://play.google.com/store/apps/details?id=com.prachakij.ldx&hl=en';
          if (await canLaunch(url)) {
            await launch(url);
          } else {
            throw 'Could not launch $url';
          }
        }
      } else {}
    }
  }
}
